#ifndef QUATERNION_H
#define QUATERNION_H

#include <ostream>
#include "Vector3.h"
#include "Matrix4x4.h"

class Quaternion {
public:
	//// Constructors ////

	// No rotation
	Quaternion();

	// Specify quaternion parameters explicitly
	Quaternion(const float u, const float i, const float j, const float k);

	// Set by rotation matrix
	Quaternion(const Matrix4x4 &m);

	// Set by Euler angles
	Quaternion(const float r, const float p, const float y);

	// Set by Euler angles
	void setAngles(const Vector3 & v);
	Quaternion(const Vector3 & v);

	// Copy
	Quaternion(const Quaternion &q);


	//// Functions ////

	// No rotation
	void reset();

	// Specify quaternion parameters explicitly
	void set(const float u, const float i, const float j, const float k);

	// Set by Euler angles
	void setAngles(const float r, const float p, const float y);

	// Set by rotation matrix
	void setMatrix(const Matrix4x4 &m);

	// Get the rotation matrix
	void getMatrix(Matrix4x4 &m) const;

	// SLERP
	void slerp(const Quaternion &q, const float f);
	void slerp(const Quaternion &q1,const Quaternion &q2, const float f);
	void lerp(const Quaternion &q, const float f);
	void lerp(const Quaternion &q1,const Quaternion &q2, const float f);

	// Return norm
	float norm();

	// Ensure norm is 1
	void normalize();

	// Cos angle between two unit quaternions
	float dot(const Quaternion &q) const;

	//calculates quat that would take this -> Q  (this->Q = C ;  this'C = Q)
	Quaternion Difference(const Quaternion & q);

	// Compose two rotations, not normalized so more than two rotations may be composed
	//   without normalizing at each step
	void rotate(const Quaternion &q);

	// Rotate a vector by this quaternion
	void rotate(Vector3 &v) const;

	// Same as rotate(), but returns a new quaternion
	Quaternion operator*(const Quaternion &q) const;

	// Copy
	Quaternion& operator=(const Quaternion &q);

	void sub(const Quaternion & q);
	void sub(const Quaternion & q1,const Quaternion & q2);
	void add(const Quaternion & q);
	void add(const Quaternion & q1,const Quaternion & q2);
	

	Quaternion inverseUnit();
	Quaternion inverse();

	void invertUnit();
	void invert();

	// Output
	friend std::ostream& operator<<(std::ostream &o, const Quaternion &q);

private:
	float a[4];	// Coefficients for the quaternion
};

#endif