#include "Material.h"
#include <SDL.h>
#include <SDL_image.h>
#include <SDLimprc.h>


std::map<std::string,Material> Material::managedMaterials;


Material::Material()
{

	//load opengl default values 
	_ambient[0] =0.2f;
	_ambient[1] =0.2f;
	_ambient[2] =0.2f;
	_ambient[3] =1.0f;

	_diffuse[0] =0.8f;
	_diffuse[1] =0.8f;
	_diffuse[2] =0.8f;
	_diffuse[3] =1.0f;

	_specular[0] =0.0f;
	_specular[1] =0.0f;
	_specular[2] =0.0f;
	_specular[3] =1.0f;

	_emissive[0] =0.0f;
	_emissive[1] =0.0f;
	_emissive[2] =0.0f;
	_emissive[3] =1.0f;
    
	_shininess = 0;    
	_textureLoaded = false;
	_textureName = -1;

}




Material::~Material()
{
	if (_textureLoaded)
		glDeleteTextures(1,&_textureName);

}



Material * 
Material::CreateManagedMaterial(const char * name)
{
	std::map<std::string,Material>::iterator it = managedMaterials.find(name);
	if (it != managedMaterials.end())
		return NULL;
	
	managedMaterials[name] = Material();
	return &managedMaterials[name];
}

void 
Material::ReleaseManagedMaterial(const char * name)
{
	std::map<std::string,Material>::iterator it = managedMaterials.find(name);
	if (it == managedMaterials.end())
		return;
	
	managedMaterials.erase(it);
}


void 
Material::ApplyManagedMaterial(const char * name)
{
	std::map<std::string,Material>::iterator it = managedMaterials.find(name);
	if (it == managedMaterials.end())
	{
		applyDefault();
		return;
	}
	it->second.apply();

}


Material * 
Material::GetManagedMaterial(const char * name)
{
	std::map<std::string,Material>::iterator it = managedMaterials.find(name);
	if (it == managedMaterials.end())
		return NULL;
	
	return &it->second;
}


void 
Material::setAmbient(float* vals)
{
	memcpy(_ambient,vals,sizeof(_ambient));
}

void 
Material::setSpecular(float* vals)
{
	memcpy(_specular,vals,sizeof(_specular));
}

void 
Material::setDiffuse(float* vals)
{
	memcpy(_diffuse,vals,sizeof(_diffuse));
}

void 
Material::setEmissive(float* vals)
{
	memcpy(_emissive,vals,sizeof(_emissive));
}

void 
Material::setShininess(float val)
{
	_shininess = val;
}


void 
Material::setAmbient(float r,float g, float b)
{
	_ambient[0] = r;
	_ambient[1] = g;
	_ambient[2] = b;
	_ambient[3] = 1.0f;
}

void 
Material::setSpecular(float r,float g, float b)
{
	_specular[0] = r;
	_specular[1] = g;
	_specular[2] = b;
	_specular[3] = 1.0f;
}

void 
Material::setDiffuse(float r,float g, float b)
{
	_diffuse[0] = r;
	_diffuse[1] = g;
	_diffuse[2] = b;
	_diffuse[3] = 1.0f;
}

void 
Material::setEmissive(float r,float g, float b)
{
	_emissive[0] = r;
	_emissive[1] = g;
	_emissive[2] = b;
	_emissive[3] = 1.0f;
}


bool 
Material::setTexture(const char * filename,bool flip)
{

	if (!filename || _textureLoaded)
		return false;
	
	//load the data
	unsigned char * data = NULL;

	SDL_Surface *image;
	
	image=IMG_Load(filename);
	if(!image) 
	{
		printf("IMG_Load: %s\n", IMG_GetError());
		return NULL;
	}
	data = ConvertImage(image,flip);	

	glGenTextures(1, &_textureName);		// Create The Texture
	glBindTexture(GL_TEXTURE_2D, _textureName); //select the texture
	glTexImage2D(GL_TEXTURE_2D, 0, 3, image->w, image->h, 0, GL_RGB, GL_UNSIGNED_BYTE, data); //load the data into the texture

	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);	// Linear Filtering
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);	// Linear Filtering
	
	SDL_FreeSurface(image);
	delete data;
	_textureLoaded = true;
	_texture = filename;
	

	return true;
}

void
Material::apply()
{
	if (_textureLoaded)
	{
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, _textureName);	
	}
	else
		glDisable(GL_TEXTURE_2D);
	
	
	glDisable(GL_COLOR_MATERIAL);
	glMaterialfv(GL_FRONT,GL_DIFFUSE,_diffuse);
	glMaterialfv(GL_FRONT,GL_AMBIENT,_ambient);
	glMaterialfv(GL_FRONT,GL_SPECULAR,_specular);
	glMaterialfv(GL_FRONT,GL_EMISSION,_emissive);
	glMaterialf(GL_FRONT,GL_SHININESS,_shininess);
}

void 
Material::applyDefault()
{
    float ambient[] = {0.2f,0.2f,0.2f,1.0f};
    float diffuse[] = {0.8f,0.8f,0.8f,1.0f};
    float specular[] = {0,0,0,1.0f};
    float emissive[] = {0,0,0,1.0f};
	
	glDisable(GL_COLOR_MATERIAL);
	glDisable(GL_TEXTURE_2D);
	glMaterialfv(GL_FRONT,GL_DIFFUSE,diffuse);
	glMaterialfv(GL_FRONT,GL_AMBIENT,ambient);
	glMaterialfv(GL_FRONT,GL_SPECULAR,specular);
	glMaterialfv(GL_FRONT,GL_EMISSION,emissive);
	glMaterialf(GL_FRONT,GL_SHININESS,0);

}
