#include <math.h>
#include "Quaternion.h"


Quaternion::Quaternion() {
	reset();
}


Quaternion::Quaternion(const float u, const float i, const float j, const float k) {
	set(u, i, j, k);
}


Quaternion::Quaternion(const Matrix4x4 &m) {
	setMatrix(m);
}


Quaternion::Quaternion(const Vector3 & v)
{
	setAngles(v.Get(0),v.Get(1),v.Get(2));
}

Quaternion::Quaternion(const float r, const float p, const float y) {
	setAngles(r, p, y);
}


Quaternion::Quaternion(const Quaternion &q) {
	a[0] = q.a[0]; a[1] = q.a[1]; a[2] = q.a[2]; a[3] = q.a[3];
}


void Quaternion::reset() {
	a[0] = 1; a[1] = a[2] = a[3] = 0;
}


void Quaternion::set(const float u, const float i, const float j, const float k) {
	a[0] = u; a[1] = i; a[2] = j; a[3] = k;
}


void
Quaternion::setAngles(const Vector3 & v)
{
	setAngles(v.Get(0),v.Get(1),v.Get(2));
}

void Quaternion::setAngles(const float r, const float p, const float y) {
	float sr, cr, sp, cp, sy, cy, spsy, cpcy;

	sr = sinf(r * 0.5f);
	cr = cosf(r * 0.5f);
	sp = sinf(p * 0.5f);
	cp = cosf(p * 0.5f);
	sy = sinf(y * 0.5f);
	cy = cosf(y * 0.5f);
	spsy = sp * sy;
	cpcy = cp * cy;

	a[0] = cr * cpcy    + sr * spsy;
	a[1] = sr * cpcy    - cr * spsy;
	a[2] = cr * sp * cy + sr * cp * sy;
	a[3] = cr * cp * sy - sr * sp * cy;
}


void Quaternion::setMatrix(const Matrix4x4 &m) {
	float trace, root;
	
	trace = m.Get(0) + m.Get(5) + m.Get(10);

	if (trace > 0.0f) {
		root = sqrtf(trace + 1.0f);
		a[0] = 0.5f * root;
		root = 0.5f / root;
		a[1] = (m.Get(9) - m.Get(6)) * root;
		a[2] = (m.Get(2) - m.Get(8)) * root;
		a[3] = (m.Get(4) - m.Get(1)) * root;
	} else {
		static const int next[3] = { 1, 2, 0 };
		int i, j, k;

		i = 0;

		if (m.Get(5) > m.Get(0))
			i = 1;
		if (m.Get(10) > m.Get(5*i))
			i = 2;

		j = next[i];
		k = next[j];

		root = sqrtf(m.Get(5*i) - m.Get(5*j) - m.Get(5*k) + 1.0f);
		a[i + 1] = 0.5f * root;
		root = 0.5f / root;
		a[0] = (m.Get(4*k + j) - m.Get(4*j + k)) * root;
		a[j + 1] = (m.Get(4*j + i) + m.Get(4*i + j)) * root;
		a[k + 1] = (m.Get(4*k + i) + m.Get(4*i + k)) * root;
	}
}


void Quaternion::getMatrix(Matrix4x4 &m) const {
	float i, j, k, ui, uj, uk, ii, ij, ik, jj, jk, kk;
	
	i = 2.0f * a[1]; j = 2.0f * a[2]; k = 2.0f * a[3];
	ui = i * a[0];   uj = j * a[0];   uk = k * a[0];
	ii = i * a[1];   ij = j * a[1];   ik = k * a[1];
	jj = j * a[2];   jk = k * a[2];   kk = k * a[3];

	m.Set(1.0f - (jj + kk), 0);
	m.Set(ij - uk,          1);
	m.Set(ik + uj,          2);
	m.Set(0,                3);
	m.Set(ij + uk,          4);
	m.Set(1.0f - (ii + kk), 5);
	m.Set(jk - ui,          6);
	m.Set(0,                7);
	m.Set(ik - uj,          8);
	m.Set(jk + ui,          9);
	m.Set(1.0f - (ii + jj), 10);
	m.Set(0,                11);
	m.Set(0,                12);
	m.Set(0,                13);
	m.Set(0,                14);
	m.Set(1,                15);
}


void Quaternion::slerp(const Quaternion& q, const float f) {
	float w1, w2, alpha, c, s;
	
	c = dot(q);

	if ((1-c) <= .000001) //very close quats. . .do lin interp
	{
		w1 = f;
		w2 = 1.0f-f;
	}
	else
	{
		alpha = acosf(fabsf(c));
		s = sinf(alpha);
		w1 = sinf((1.0f - f) * alpha) / s;
		w2 = sinf(f * alpha) / s;
	
		if (c < 0.0f)
			w1 = -w1;
	}
	a[0] = w1*a[0] + w2*q.a[0];
	a[1] = w1*a[1] + w2*q.a[1];
	a[2] = w1*a[2] + w2*q.a[2];
	a[3] = w1*a[3] + w2*q.a[3];
}


void 
Quaternion::lerp(const Quaternion& q, const float f) 
{
	float w1 = f;
	float w2 = 1.0f-f;
	a[0] = w1*a[0] + w2*q.a[0];
	a[1] = w1*a[1] + w2*q.a[1];
	a[2] = w1*a[2] + w2*q.a[2];
	a[3] = w1*a[3] + w2*q.a[3];
	normalize();
}

void 
Quaternion::lerp(const Quaternion &q1,const Quaternion &q2, const float f)
{
	float w1 = f;
	float w2 = 1.0f-f;
	a[0] = w1*q1.a[0] + w2*q2.a[0];
	a[1] = w1*q1.a[1] + w2*q2.a[1];
	a[2] = w1*q1.a[2] + w2*q2.a[2];
	a[3] = w1*q1.a[3] + w2*q2.a[3];
	normalize();
}


void 
Quaternion::slerp(const Quaternion &q1,const Quaternion &q2, const float f)
{
	float w1, w2, alpha, c, s;
	
	c = q1.dot(q2);

	if ((1-c) <= .000001) //very close quats. . .do lin interp
	{
		w1 = f;
		w2 = 1.0f-f;
	}
	else
	{
		alpha = acosf(fabsf(c));
		s = sinf(alpha);
		w1 = sinf((1.0f - f) * alpha) / s;
		w2 = sinf(f * alpha) / s;
	
		if (c < 0.0f)
			w1 = -w1;
	}
	a[0] = w1*q1.a[0] + w2*q2.a[0];
	a[1] = w1*q1.a[1] + w2*q2.a[1];
	a[2] = w1*q1.a[2] + w2*q2.a[2];
	a[3] = w1*q1.a[3] + w2*q2.a[3];
}

void 
Quaternion::sub(const Quaternion & q)
{
	a[0] -= q.a[0];
	a[1] -= q.a[1];
	a[2] -= q.a[2];
	a[3] -= q.a[3];
}

void 
Quaternion::sub(const Quaternion & q1,const Quaternion & q2)
{
	a[0] = q1.a[0]-q2.a[0];
	a[1] = q1.a[1]-q2.a[1];
	a[2] = q1.a[2]-q2.a[2];
	a[3] = q1.a[3]-q2.a[3];
}

void 
Quaternion::add(const Quaternion & q)
{
	a[0] += q.a[0];
	a[1] += q.a[1];
	a[2] += q.a[2];
	a[3] += q.a[3];
}

void 
Quaternion::add(const Quaternion & q1,const Quaternion & q2)
{
	a[0] = q1.a[0]+q2.a[0];
	a[1] = q1.a[1]+q2.a[1];
	a[2] = q1.a[2]+q2.a[2];
	a[3] = q1.a[3]+q2.a[3];
}


float Quaternion::norm() {
	return sqrtf(a[0]*a[0] + a[1]*a[1] + a[2]*a[2] + a[3]*a[3]);
}


void Quaternion::normalize() {
	float n = norm();
	a[0] /= n; a[1] /= n; a[2] /= n; a[3] /= n;
}


float Quaternion::dot(const Quaternion &q) const {
	return a[0]*q.a[0] + a[1]*q.a[1] + a[2]*q.a[2] + a[3]*q.a[3];
}



void Quaternion::rotate(const Quaternion &q) {
	(*this) = Quaternion(a[0]*q.a[0] - a[1]*q.a[1] - a[2]*q.a[2] - a[3]*q.a[3],
	                  a[0]*q.a[1] + a[1]*q.a[0] + a[2]*q.a[3] - a[3]*q.a[2],
	                  a[0]*q.a[2] - a[1]*q.a[3] + a[2]*q.a[0] + a[3]*q.a[1],
	                  a[0]*q.a[3] + a[1]*q.a[2] - a[2]*q.a[1] + a[3]*q.a[0]);
}


void Quaternion::rotate(Vector3 &v) const {
	float a00, a01, a02, a03, a11, a12, a13, a22, a23, v0, v1, v2;

	a00 = 2.0f * a[0] * a[0];
	a01 = 2.0f * a[0] * a[1];
	a02 = 2.0f * a[0] * a[2];
	a03 = 2.0f * a[0] * a[3];
	a11 = 2.0f * a[1] * a[1];
	a12 = 2.0f * a[1] * a[2];
	a13 = 2.0f * a[1] * a[3];
	a22 = 2.0f * a[2] * a[2];
	a23 = 2.0f * a[2] * a[3];

	v0 = (1.0f-a11-a22) * v.Get(0) +      (a01-a23) * v.Get(1) +      (a02+a13) * v.Get(2);
	v1 =      (a01+a23) * v.Get(0) + (1.0f-a22-a00) * v.Get(1) +      (a12-a03) * v.Get(2);
	v2 =      (a02-a13) * v.Get(0) +      (a12+a03) * v.Get(1) + (1.0f-a11-a00) * v.Get(2);

	v.Set(v0, v1, v2);
}


Quaternion Quaternion::operator*(const Quaternion &q) const {
	return Quaternion(a[0]*q.a[0] - a[1]*q.a[1] - a[2]*q.a[2] - a[3]*q.a[3],
	                  a[0]*q.a[1] + a[1]*q.a[0] + a[2]*q.a[3] - a[3]*q.a[2],
	                  a[0]*q.a[2] - a[1]*q.a[3] + a[2]*q.a[0] + a[3]*q.a[1],
	                  a[0]*q.a[3] + a[1]*q.a[2] - a[2]*q.a[1] + a[3]*q.a[0]);
}


Quaternion& Quaternion::operator=(const Quaternion &q) {
	a[0] = q.a[0]; a[1] = q.a[1]; a[2] = q.a[2]; a[3] = q.a[3];
	return (*this);
}


std::ostream& operator<<(std::ostream &o, const Quaternion &q) {
	return o << q.a[0] << " + " << q.a[1] << "i + " << q.a[2] << "j + " << q.a[3] << 'k';
}

Quaternion 
Quaternion::inverseUnit()
{
	return Quaternion(a[0],-a[1],-a[2],-a[3]);
}

Quaternion 
Quaternion::inverse()
{
	float norm = a[0]*a[0] + a[1]*a[1] + a[2]*a[2] + a[3]*a[3];
	if (norm > 0)
	{
		float norminv = 1.0f/norm;
		return Quaternion(a[0]*norminv,-a[1]*norminv,-a[2]*norminv,-a[3]*norminv);
	}
	return Quaternion();
}

Quaternion 
Quaternion::Difference(const Quaternion & q)
{
	Quaternion ret = inverse();
	ret.rotate(q);
	return ret;
}

void 
Quaternion::invertUnit()
{
	a[1] *= -1;
	a[2] *= -1;
	a[3] *= -1;
}

void 
Quaternion::invert()
{
	float norm = a[0]*a[0] + a[1]*a[1] + a[2]*a[2] + a[3]*a[3];
	if (norm > 0)
	{
		float norminv = 1.0f/norm;
		a[0] *= norminv;
		a[1] *= -norminv;
		a[2] *= -norminv;
		a[3] *= -norminv;
	}
	
}