#include "RenderableObject.h"

int RenderableObject::nextID = 0;

//not thread safe
RenderableObject::RenderableObject()
{
	Scale.Set(1.0f,1.0f,1.0f);
	objID = nextID;
	nextID++; 
	invDirty = false;
	parent = NULL;
}

RenderableObject::~RenderableObject()
{
}

void RenderableObject::AddChild(RenderableObject*r)
{
	Children[r->objID] = r;
	r->UpdateParent(this);
}


void 
RenderableObject::UpdateParent(RenderableObject * p)
{
	if (parent)
		parent->RemoveChild(objID);
	parent = p;
}

void 
RenderableObject::SetAngle(const Quaternion & q)
{
	Angle = q;
}

void 
RenderableObject::SetAngle(const Vector3 & v)
{
	Angle.setAngles(v);
}
void 
RenderableObject::SetPos(const Vector3 & v)
{
	Pos = v;
}
void 
RenderableObject::SetScale(const Vector3 & v)
{
	Scale = v;
}

void RenderableObject::RemoveChild(int objId)
{
	Children.erase(objId);
}

void RenderableObject::Render()
{
	glPushMatrix();
	ApplyTransform();
	Draw();
	for(std::map<int, RenderableObject*>::iterator i = Children.begin(); i != Children.end(); i++)
		i->second->Render();

	glPopMatrix();
}


bool RenderableObject::operator < (const RenderableObject & obj) const 
{
	return (objID < obj.objID);
}

int RenderableObject::GetID()const
{
	return objID;
}
void RenderableObject::ApplyTransform()
{
	//apply transformation matrix
	glTranslatef(Pos.Get(0),Pos.Get(1),Pos.Get(2));
	Matrix4x4 m;
	Angle.getMatrix(m);
	m.Transpose();
	glMultMatrixf(m.GetPtr());
	glScalef(Scale.Get(0),Scale.Get(1),Scale.Get(2));
	glMultMatrixf(xform.GetPtr());
}

void RenderableObject::SetXform(const Matrix4x4 & m)
{
	xform = m;
	xform.Transpose();
	invDirty = true;
}

void RenderableObject::ApplyInverseTransform()
{
	//apply the inverse transformation matrix

	if (invDirty)
	{
		xform.Transpose();
		if (!xform.GetInverse(xforminv))
			xforminv.Identity();
		xform.Transpose();
		xforminv.Transpose();
		invDirty = false;
	}

	glMultMatrixf(xforminv.GetPtr());
	glScalef(1.0f/Scale.Get(0),1.0f/Scale.Get(1),1.0f/Scale.Get(2));
	Matrix4x4 m;
	Angle.getMatrix(m);
	glMultMatrixf(m.GetPtr());
	glTranslatef(-Pos.Get(0),-Pos.Get(1),-Pos.Get(2));

	if (parent)
		parent->ApplyInverseTransform();

}


void RenderableObject::SetPos(GLfloat x,GLfloat y,GLfloat z)
{
	//printf("Object %d moved to (%f , %f , %f)\n",objID,x,y,z);

	Pos.Set(x,y,z);
}

void RenderableObject::SetAngle(GLfloat x, GLfloat y,GLfloat z)
{
	//printf("Object %d rotated to (%f , %f , %f)\n",objID,x,y,z);
	Angle.setAngles(x,y,z);
}

void RenderableObject::SetScale(GLfloat x, GLfloat y,GLfloat z)
{
	//printf("Object %d scaled to (%f , %f , %f)\n",objID,x,y,z);
	Scale.Set(x,y,z);
}


 void RenderableObject::Draw()
{

	//a point!
	glBegin(GL_POINTS);
		glColor3f(1,0,0);
		glVertex3f(0,0,0);
	glEnd();
}

void 
RenderableObject::GetFullXform(Matrix4x4 & m)
{
	//apply transformation matrix
	Matrix4x4 t,r,s,p;
	
	if (parent)
		parent->GetFullXform(p);

	t.Translate(Pos);
	Angle.getMatrix(r);
	s.Scale(Scale);

	m = xform;
	m.Transpose();

	m.PreMult(s);
	m.PreMult(r);
	m.PreMult(t);
	
	if (parent)
		m.PreMult(p);
}

void 
RenderableObject::GetLocalXform(Matrix4x4 & m)
{
	//apply transformation matrix
	Matrix4x4 t,r,s,p;
	
	t.Translate(Pos);
	Angle.getMatrix(r);
	s.Scale(Scale);

	m = xform;
	m.Transpose();

	m.PreMult(s);
	m.PreMult(r);
	m.PreMult(t);

}


void 
RenderableObject::GetPos(Vector3 & v) const
{
	v = Pos;
}


void 
RenderableObject::GetFullXformInv(Matrix4x4 & m)
{

	Matrix4x4 t,r,s,p;
	
	if (parent)
		parent->GetFullXformInv(p);

	t.Translate(-Pos.Get(0),-Pos.Get(1),-Pos.Get(2));
	Angle.getMatrix(r);
	r.Transpose();
	s.Scale(1.0f/Scale.Get(0),1.0f/Scale.Get(1),1.0f/Scale.Get(2));

	if (parent)
		m.Mult(t,p);
	else
		m = t;
	m.PreMult(r);
	m.PreMult(s);
	Matrix4x4 xfi = xforminv;
	xfi.Transpose();
	m.PreMult(xfi);
}
