#define REQUIRE_IOSTREAM
#include <maya/MSimple.h>
#include <maya/MStatus.h>
#include <maya/MString.h> 
#include <maya/MFileIO.h>
#include <maya/MGlobal.h>
#include <maya/MLibrary.h>
#include <maya/MIOStream.h>
#include <maya/MItDag.h>
#include <maya/MDagPath.h>
#include <maya/MPointArray.h>
#include <maya/MFnDagNode.h>
#include <maya/MFnMesh.h>
#include <maya/MItMeshPolygon.h>
#include <maya/MItDependencyNodes.h>
#include <maya/MItDependencyGraph.h>
#include <maya/MFnSkinCluster.h>
#include <maya/MDagPathArray.h>
#include <maya/MItGeometry.h>
#include <maya/MFnIKJoint.h>
#include <maya/MEulerRotation.h>
#include <maya/MTime.h>
#include <maya/MFnAnimCurve.h>
#include <maya/MAnimControl.h>
#include <maya/MSelectionList.h>
#include <maya/MItSelectionList.h>
#include <maya/MMatrix.h>
#include <maya/MFnMatrixData.h>
#include <maya/MFnAttribute.h>
#include <maya/MAnimUtil.h>

#include <set>
#include <map>
#include <matrix4x4.h>
#include <vector3.h>

#define MAXBONES 4

using namespace std;

DeclareSimpleCommand(skelExport, "Alias|Wavefront", "5.0");



MMatrix printDagPoseInfo(MObject& dagPoseNode, unsigned index)
//
// Description:
//   Given a dagPose and an index corresponding to a joint, print out
//   the matrix info for the joint.
// Return:
//	 None.
//
{

	MFnDependencyNode nDagPose(dagPoseNode);

	// construct plugs for this joints world and local matrices
	//
	MObject aWorldMatrix = nDagPose.attribute("worldMatrix");
	MPlug pWorldMatrix(dagPoseNode,aWorldMatrix);
	pWorldMatrix.selectAncestorLogicalIndex(index,aWorldMatrix);
	
	// get and print the world matrix data
	//
	MObject worldMatrix, xformMatrix;
	MStatus status = pWorldMatrix.getValue(worldMatrix);

	if (MS::kSuccess != status) 
	{
	} else {
		bool foundMatrix = 0;
		MFnMatrixData dMatrix(worldMatrix);
		MMatrix wMatrix = dMatrix.matrix(&status);
		if (MS::kSuccess == status) {
			return wMatrix;
			//return wMatrix.inverse();
			}
		}
	return MMatrix();
}



MMatrix getInvBindPose(MObject& jointNode)
//
// Description:
//   Given a joint, check for connected dag pose nodes.
//   For each pose found, write out the pose info.
// Return:
//	 If one or more poses is found, return true, else return false.
//
{
	bool rtn = 0; // return 1 if we find a pose

	MStatus status;
	MFnDependencyNode fnJoint(jointNode);
	MObject aBindPose = fnJoint.attribute("bindPose",&status);

	if (MS::kSuccess == status) {
		unsigned connLength = 0;
		MPlugArray connPlugs;
		MPlug pBindPose(jointNode,aBindPose);
		pBindPose.connectedTo(connPlugs,false,true);
		connLength = connPlugs.length();
		for (unsigned ii = 0; ii < connLength; ++ii) {
			if (connPlugs[ii].node().apiType() == MFn::kDagPose) {
				MObject aMember = connPlugs[ii].attribute();
				MFnAttribute fnAttr(aMember);
				if (fnAttr.name() == "worldMatrix") {
					unsigned jointIndex = connPlugs[ii].logicalIndex();

					MObject jointObject = connPlugs[ii].node();
					return printDagPoseInfo(jointObject,jointIndex).inverse();
					
				}
			}
		}
	}
	return MMatrix();
}






typedef struct 
{
	float rot[3];
	float trans[3];
}xform;

bool cmpXform(xform & x1, xform & x2)
{
	return ( (x1.rot[0] == x2.rot[0]) 
		&&(x1.rot[1] == x2.rot[1]) 
		&&(x1.rot[2] == x2.rot[2])); 
}

struct joint
{
	char name[20];
	float relpos[3];
	float orient[3];
	int parent;
	Matrix4x4 inv;
	MString mj;
	std::set<float> keys;
};

struct vertex
{
	float pos[3];
	int bone[MAXBONES];
	float weights[MAXBONES];
	int numbones;
};

struct triangle
{
	int norms[3];
	int verts[3];
	int texcoord[3];

};

struct texcoord
{
	float uv[2];
};
struct normal
{
	float orient[3];
};

struct skelchar
{
	normal * norms;
	vertex * verts;
	triangle * polys;
	joint * joints;
	texcoord * uvs;
	int numverts;
	int numpolys;
	int numjoints;
	int numnorms;
	int numtexcoords;
	std::set<float> transKeys;
};

std::set<float>  findFrames(float mintime, float maxtime)
{
	std::set<float> keytimes;
	keytimes.insert(mintime);
	keytimes.insert(maxtime);
	MStatus stat;
	MItDag dagIter( MItDag::kBreadthFirst, MFn::kInvalid, &stat );
	MTime maxKeyTime;
	maxKeyTime.setValue(0);

	for ( ; !dagIter.isDone(); dagIter.next())
	{
		MDagPath dagPath;
		stat = dagIter.getPath( dagPath );

		MFnDagNode dn( dagPath );

		// find all the motion nodes
		MItDependencyGraph dgIter( dagPath.node(),
			MFn::kAnimCurve,
			MItDependencyGraph::kUpstream,
			MItDependencyGraph::kBreadthFirst,
			MItDependencyGraph::kNodeLevel,
			&stat );
		if ( stat )
		{
			// get all the keyframe times
			for ( ; !dgIter.isDone(); dgIter.next())
			{
				MObject anim = dgIter.thisNode( &stat );
				MFnAnimCurve animCurve( anim, &stat );
				// found some keyframe information
				if ( MS::kSuccess == stat )
				{
					unsigned int numKeys = animCurve.numKeyframes( &stat );
					for ( unsigned int currKey = 0; currKey < numKeys; currKey++ )
					{
						// truncating values here...
						MTime keytime = animCurve.time( currKey, &stat );
						if ((keytime.as(MTime::kMilliseconds) >= mintime) && (keytime.as(MTime::kMilliseconds)<= maxtime))
						{
							keytimes.insert((float) keytime.as(MTime::kMilliseconds));

						}
					}
				}
			}
		}
	}
	keytimes.insert(mintime);
	keytimes.insert(maxtime);
	return keytimes;
}




void writeAnim(skelchar * skel, const char * fname)
{
	FILE * f = fopen(fname,"wb");
	MAnimControl animctrl;

	fwrite("ANIM",1,5,f);
	//write out the number of joints
	fwrite(&skel->numjoints,4,1,f);
	
	//evaluate and write out all joints
	for (int i = 0; i < skel->numjoints; i++)
	{
		
		//write out joint name
		fwrite(skel->joints[i].name,1,21,f);
		
		//write out number of keys
		unsigned int numkeys = skel->joints[i].keys.size();
		fwrite(&numkeys,4,1,f);

		//write out the keys and the rotation values
		for (std::set<float>::iterator it = skel->joints[i].keys.begin(); it != skel->joints[i].keys.end(); it++)
		{

			//set the time
			MTime time;
			time.setUnit(MTime::kMilliseconds);
			time.setValue(*it);
			animctrl.setCurrentTime(time);

			//write out the time
			float t = *it - *skel->joints[i].keys.begin();
			fwrite(&t,4,1,f);

			//get the rotation value of the current joint
			MSelectionList select;
			MGlobal::selectByName(skel->joints[i].mj,	MGlobal::kReplaceList);
			MGlobal::getActiveSelectionList(select);
			MObject jnt;
			select.getDependNode(0,jnt);
			MFnIkJoint mj(jnt);
			MEulerRotation rot;
			mj.getRotation(rot);
			float r[3];
			r[0] = (float)rot.x;
			r[1] = (float)rot.y;
			r[2] = (float)rot.z;
			
			//write out the rotation value
			fwrite(r,4,3,f);
		}
	}

	//now write out the translations for the root
	//write out number of keys
	unsigned int numkeys = skel->joints[0].keys.size();
	fwrite(&numkeys,4,1,f);

	//write out the keys 
	float timeZero = *skel->joints[0].keys.begin();
	for (std::set<float>::iterator it = skel->transKeys.begin(); it != skel->transKeys.end(); it++)
	{
		//set the time
		MTime time;
		time.setUnit(MTime::kMilliseconds);
		time.setValue(*it);
		animctrl.setCurrentTime(time);


		//write out the time
		float t = *it - timeZero;
		fwrite(&t,4,1,f);

		MSelectionList select;
		MGlobal::selectByName(skel->joints[0].mj,	MGlobal::kReplaceList);
		MGlobal::getActiveSelectionList(select);
		MObject jnt;
		select.getDependNode(0,jnt);
		MFnIkJoint mj(jnt);
		

		MVector v = mj.translation(MSpace::kPostTransform);
		float tr[3];
		tr[0] = (float)v.x;
		tr[1] = (float)v.y;
		tr[2] = (float)v.z;
		printf("%f: (%f,%f,%f)\n",t,tr[0],tr[1],tr[2]);
		fwrite(tr,4,3,f);
	}



	fclose(f);
}


/*
void writeAnim(skelchar * skel, std::set<float> times, const char * fname)
{
	FILE * f = fopen(fname,"wb");
	MAnimControl animctrl;

	float minTime = 10000000;

	
	printf("%d Key frames found in range\n",times.size());
	for(std::set<float>::iterator i = times.begin(); i != times.end(); i++)
	{
		//printf("Exporting key at time %f\n",*i);
		MTime time;
		time.setUnit(MTime::kMilliseconds);
		time.setValue(*i);
		animctrl.setCurrentTime(time);

		minTime = (float) min(time.as(MTime::kMilliseconds),minTime);

		Vector3 Ts;

		for(int j = 0; j < skel->numjoints; j++)
		{
			MSelectionList select;
			MGlobal::selectByName(skel->joints[j].mj,	MGlobal::kReplaceList);
			MGlobal::getActiveSelectionList(select);

			MObject jnt;
			select.getDependNode(0,jnt);

			MFnIkJoint mj(jnt);
			MEulerRotation rot;
			mj.getRotation(rot);
			xform x;
			x.rot[0] = (float) rot.x;
			x.rot[1] = (float) rot.y;
			x.rot[2] = (float) rot.z;
			
			MVector v = mj.translation(MSpace::kObject);
			x.trans[0] = (float)v.x;
			x.trans[1] = (float)v.y;
			x.trans[2] = (float)v.z;

			skel->joints[j].keys[(float) time.as(MTime::kMilliseconds)] = x;

		}
	}

	int numkeys = times.size() * skel->numjoints;

	fwrite(&numkeys,1,4,f);
	fwrite(&skel->numjoints,1,4,f);

	//write out the rotation keys
	int keys = 0;
	for (int j = 0; j < skel->numjoints; j++)
	{
		for (std::map<float,xform>::iterator i = skel->joints[j].keys.begin(); i != skel->joints[j].keys.end(); i++)
		{
			//joint num
			fwrite(&j,4,1,f);
			float time = i->first - minTime;
			xform & x = i->second;

			//time
			fwrite(&time,4,1,f);

			//rotation
			fwrite(&x.rot,4,3,f);

			//translation
			fwrite(&x.trans,4,3,f);

			keys++;
		}

	}

	fclose(f);
	printf("%d Animation Keys written to %s\n",keys,fname);

}
*/

void writeSkel(const char * filename, skelchar * skel)
{//sz cnt
	FILE * f = fopen(filename,"wb");
	fwrite("SKEL",1,5,f);
	fwrite(&skel->numjoints,4,1,f);
	fwrite(&skel->numverts,4,1,f);
	fwrite(&skel->numpolys,4,1,f);
	fwrite(&skel->numnorms,4,1,f);
	fwrite(&skel->numtexcoords,4,1,f);

	for (int i = 0; i < skel->numjoints;i++)
	{
		fwrite(skel->joints[i].name,1,21,f);
		fwrite(skel->joints[i].relpos,4,3,f);
		fwrite(skel->joints[i].orient,4,3,f);
		fwrite(&skel->joints[i].parent,4,1,f);
		
		float mat[16];
		skel->joints[i].inv.GetAll(mat);
		fwrite(mat,4,16,f);
	}

	for (int i = 0; i < skel->numverts;i++)
	{
		fwrite(skel->verts[i].pos,4,3,f);
		fwrite(&skel->verts[i].numbones,4,1,f);
		fwrite(skel->verts[i].bone,4,skel->verts[i].numbones,f);
		fwrite(skel->verts[i].weights,4,skel->verts[i].numbones,f);
	}

	for (int i = 0; i < skel->numpolys;i++)
	{
		fwrite(skel->polys[i].verts,4,3,f);
		fwrite(skel->polys[i].norms,4,3,f);
		fwrite(skel->polys[i].texcoord,4,3,f);
		
	}

	for (int i = 0; i < skel->numnorms;i++)
	{
		fwrite(skel->norms[i].orient,4,3,f);
	}

	for (int i = 0; i < skel->numtexcoords;i++)
	{
		fwrite(skel->uvs[i].uv,4,2,f);
	}

	fclose(f);
	printf("Skeleton Written to %s: %dj %dv %dp %dn\n",filename,skel->numjoints,skel->numverts,skel->numpolys,skel->numnorms);

}

void fillJointParams(MFnSkinCluster & skin, MFnIkJoint& j, joint * thejoint)
{
	MStatus stat;
	strncpy(thejoint->name,j.name().asChar(),sizeof(thejoint->name)-1);
	MDagPath path = MDagPath::getAPathTo(j.parent(0));
	int pindex = skin.indexForInfluenceObject(path,&stat);
	if (stat == MStatus::kSuccess)
		thejoint->parent = pindex;
	else
		thejoint->parent = -1;

	MEulerRotation er;
	j.getOrientation(er);

	MVector rot = er.asVector();
	thejoint->orient[0] = (float)rot.x;
	thejoint->orient[1] = (float)rot.y;
	thejoint->orient[2] = (float)rot.z;

	//MVector pos =j.translation(MSpace::kPreTransform);
	MVector pos =j.translation(MSpace::kPostTransform);
	thejoint->relpos[0] = (float)pos.x;
	thejoint->relpos[1] = (float)pos.y;
	thejoint->relpos[2] = (float)pos.z;
	
	MMatrix inv = getInvBindPose(j.object());

	for(int r = 0; r < 4;r++)
		for(int c = 0; c < 4;c++)
			thejoint->inv.Set(inv(r,c),r*4+c);
}

MObject getSkinClusterForJoint(MObject & j)
{
	MStatus stat;
	MItDependencyNodes itNodes(MFn::kSkinClusterFilter);
	for(/**/;!itNodes.isDone();itNodes.next())
	{
		MFnSkinCluster fnSkin(itNodes.item());

		MDagPathArray infObjArray;
		fnSkin.influenceObjects(infObjArray,&stat);
		for(uint i = 0; i<infObjArray.length();i++)
			if (infObjArray[i].node() == j)
				return itNodes.item();

	}
	return MObject::kNullObj;
}


/*
void getDeformedMesh(MFnMesh & meshFn)
{
	MStatus stat;
	MFnDagNode dagNode(meshDagPath); // path to the visible mesh
	MFnMesh meshFn(dagPath, &stat); // this is the visible mesh
	MObject inObj;
	MObject dataObj1; 

	// the deformed mesh comes into the visible mesh
	// through its "inmesh" plug
	MPlug inMeshPlug = dagNode.findPlug("inMesh", &stat);

	if (stat == MS::kSuccess && inMeshPlug.isConnected())
	{
		// walk the tree of stuff upstream from this plug
		MItDependencyGraph dgIt(inMeshPlug,
			MFn::kInvalid,
			MItDependencyGraph::kUpstream,
			MItDependencyGraph::kDepthFirst,
			MItDependencyGraph::kPlugLevel,
			&stat);

		if (MS::kSuccess == stat)
		{
			dgIt.disablePruningOnFilter();
			int count = 0;

			for ( ; ! dgIt.isDone(); dgIt.next() )
			{
				MObject thisNode = dgIt.thisNode();

				// go until we find a skinCluster

				if (thisNode.apiType() == MFn::kSkinClusterFilter)
				{
					MFnSkinCluster skinCluster(thisNode);

					// get the mesh coming into the skinCluster. This
					// is the mesh before being deformed but after
					// being edited/tweaked/etc.

					MPlug inputPlug = skinCluster.findPlug("input", &stat); 
					if (stat == MS::kSuccess)
					{
						MPlug childPlug = inputPlug.elementByLogicalIndex(0); 
						MPlug geomPlug = childPlug.child(0); 

						geomPlug.getValue(dataObj1); 

						// let use this mesh instead of the visible one
						meshFn.setObject(dataObj1); 
					}
				}
			}
		}
	}
}
*/


bool findTimesByPlug(MFnDagNode & dagNode, const char * plugname, std::set<float> & times,float minTime, float maxTime)
{
	MStatus stat;
	MPlug plug = dagNode.findPlug(plugname, &stat);
	if (stat != MStatus::kSuccess)
		return false;
	MObjectArray arr;

	times.insert(minTime);
	times.insert(maxTime);

	if (MAnimUtil::findAnimation(plug,arr))
	{
		MFnAnimCurve animCurve(arr[0]);
		for (unsigned int i = 0; i < animCurve.numKeys();i++)
		{
			MTime time = animCurve.time(i);
			float t = (float)time.as(MTime::kMilliseconds);
			if ((t < maxTime) && (t > minTime))
				times.insert(t);
		}
	}

	return true;
}




skelchar * loadSkeleton(const MString rootname, const float minTime, const float maxTime)
{
	//find the object with rootname as its name
	MSelectionList select;
	MGlobal::selectByName(rootname,	MGlobal::kReplaceList);
	MGlobal::getActiveSelectionList(select);
	if (select.isEmpty())
	{
		cout << "No object '" << rootname.asChar() << "' in scene" << endl;
		return NULL;
	}

	//make sure rootname is a joint
	MObject root;
	select.getDependNode(0,root);
	if (!root.hasFn(MFn::kJoint))
	{
		cout << "Object '" << rootname.asChar() << "' is not a joint" << endl;
		return NULL;
	}

	//get the skincluster object associated with the joint
	MFnIkJoint rj(root);
	MObject skinClusterObj = getSkinClusterForJoint(root);
	if (skinClusterObj.isNull())
	{
		cout << "Skeleton not bound to geometry" << endl;
		return NULL;
	}
	MFnSkinCluster fnSkin(skinClusterObj);


	MStatus stat;

	//get the full set of joints
	MDagPathArray infObjArray;
	fnSkin.influenceObjects(infObjArray,&stat);

	skelchar * skel = new(skelchar);

	//get the mesh object
	MObjectArray meshArray;
	fnSkin.getInputGeometry(meshArray);
	MFnMesh fnMesh(meshArray[0]);
	
	//load the poly info
	skel->numpolys = fnMesh.numPolygons();
	skel->polys = new(triangle[skel->numpolys]);

	for (unsigned int i = 0; i < (unsigned int) skel->numpolys; i++)
	{
		MIntArray vertlist;
		MIntArray norms;

		fnMesh.getFaceNormalIds(i,norms);
		fnMesh.getPolygonVertices(i,vertlist);


		if (vertlist.length() != 3)
		{
			cout << "Non Triangle Found" << endl;
			delete(skel->polys);
			delete(skel);			
			return NULL;
		}

		fnMesh.getPolygonUVid(i,0,skel->polys[i].texcoord[0]);
		fnMesh.getPolygonUVid(i,1,skel->polys[i].texcoord[1]);
		fnMesh.getPolygonUVid(i,2,skel->polys[i].texcoord[2]);

		skel->polys[i].verts[0] = vertlist[0];
		skel->polys[i].verts[1] = vertlist[1];
		skel->polys[i].verts[2] = vertlist[2];
		skel->polys[i].norms[0] = norms[0];
		skel->polys[i].norms[1] = norms[1];
		skel->polys[i].norms[2] = norms[2];

	}

	//load the normal info
	skel->numnorms = fnMesh.numNormals();
	MFloatVectorArray norms;
	fnMesh.getNormals(norms);
	skel->norms = new(normal[skel->numnorms]);
	for (unsigned int i = 0; i < (unsigned int) skel->numnorms; i++)
	{
		skel->norms[i].orient[0] = norms[i].x;
		skel->norms[i].orient[1] = norms[i].y;
		skel->norms[i].orient[2] = norms[i].z;
	}

	skel->numtexcoords = fnMesh.numUVs();

	MFloatArray u, v;
	fnMesh.getUVs(u,v);
	skel->uvs = new(texcoord[skel->numtexcoords]);
	for (unsigned int i = 0; i < (unsigned int) skel->numtexcoords; i++)
	{
		skel->uvs[i].uv[0] = u[i];
		skel->uvs[i].uv[1] = v[i];
	}



	//load the vertex info
	MPointArray vertexList;
	fnMesh.getPoints( vertexList, MSpace::kObject);
	skel->verts = new(vertex[vertexList.length()]);
	skel->numverts = vertexList.length();

	for (unsigned int i = 0; i < vertexList.length(); i++)
	{
		vertexList[i].cartesianize();

		skel->verts[i].pos[0] = (float) vertexList[i].x;
		skel->verts[i].pos[1] = (float) vertexList[i].y;
		skel->verts[i].pos[2] = (float) vertexList[i].z;
		skel->verts[i].numbones = 0;
	}



	skel->joints = new (joint[infObjArray.length()]);
	skel->numjoints = infObjArray.length();

	//get the translation keyframe times
	MFnIkJoint rjnt(infObjArray[0]);
	findTimesByPlug(rjnt,"translateX",skel->transKeys,minTime,maxTime);
	findTimesByPlug(rjnt,"translateY",skel->transKeys,minTime,maxTime);
	findTimesByPlug(rjnt,"translateZ",skel->transKeys,minTime,maxTime);

	//loop through all joints
	for(uint i = 0; i<infObjArray.length();i++)
	{        
		//fill out the joint structure
		MFnIkJoint jnt(infObjArray[i]);
		
		fillJointParams(fnSkin,jnt,&skel->joints[i]);

		findTimesByPlug(jnt,"rotateX",skel->joints[i].keys,minTime,maxTime);
		findTimesByPlug(jnt,"rotateY",skel->joints[i].keys,minTime,maxTime);
		findTimesByPlug(jnt,"rotateZ",skel->joints[i].keys,minTime,maxTime);


		skel->joints[i].mj = jnt.fullPathName();

		MSelectionList result;
		MFloatArray weights;
		fnSkin.getPointsAffectedByInfluence (infObjArray[i], result,  weights);

		MDagPath mesh;
		MObject vertices;
		result.getDagPath(0,mesh,vertices);
		MItGeometry itGeom(mesh,vertices);
		int len = weights.length();
		for(uint k = 0;!itGeom.isDone();itGeom.next(),k++)
		{
			int index = itGeom.index();
			skel->verts[index].bone[skel->verts[index].numbones] = i;
			skel->verts[index].weights[skel->verts[index].numbones] = weights[k];
			skel->verts[index].numbones++;
			if (skel->verts[index].numbones > MAXBONES)
			{
				cout << "too many joints per vertex!" << endl;

				delete skel->joints;
				delete skel->norms;
				delete skel->polys;
				delete skel->verts;
				delete skel->uvs;
				delete skel;
				return NULL;
			}
		}
	}
	

	return skel;
}


MStatus skelExport::doIt( const MArgList& args )
{
	MStatus stat;
	MString arg, rootJoint, outDir,tmp,skelfile,animfile;
	MSelectionList list;
	MObject component;
    MDagPath node;
	MFnDagNode nodeFn;
	int startFrame, endFrame;
	startFrame = 1;
	endFrame = 24;
	MGlobal::getActiveSelectionList( list );

	skelfile = "test.skel";
	animfile = "test.anim";
	rootJoint = "root";	// Default to look for 'root' if none selected or named
	outDir = "c:/temp/";

	// Look for last selected joint
	MItSelectionList listIter( list, MFn::kJoint );
	for ( ; !listIter.isDone(); listIter.next())
	{
		listIter.getDagPath( node, component );
		nodeFn.setObject( node );
		rootJoint = nodeFn.name();
	}

	// Parse arguments
	for ( unsigned int i = 0; i < args.length(); i++ )
	{
		arg = args.asString( i, &stat );
		if (!stat)              
			continue;
		
		if ( arg == "-s" )
		{
			if (i == args.length()-1)
			{
				arg += ": must specify the skeleton file to export";
				displayError(arg);
				return MS::kFailure;
			}
			++i;
			args.get(i, skelfile);
		}

		else if ( arg == "-a" )
		{
			if (i == args.length()-1)
			{
				arg += ": must specify the anim file to export";
				displayError(arg);
				return MS::kFailure;
			}
			++i;
			args.get(i, animfile);
		}
				
		else if ( arg == "-r" || arg == "-root" )
		{
			if (i == args.length()-1)
			{
				arg += ": must specify a root node of the skeleton";
				displayError(arg);
				return MS::kFailure;
			}
			++i;
			args.get(i, rootJoint);
		}
		else if ( arg == "-o" || arg == "-outdir" )
		{
			if (i == args.length()-1)
			{
				arg += ": must specify an output directory";
				displayError(arg);
				return MS::kFailure;
			}
			++i;
			args.get(i, outDir);
		}
		else if ( arg == "-start" )
		{
			if (i == args.length()-1)
			{
				arg += ": must specify a start frame";
				displayError(arg);
				return MS::kFailure;
			}
			++i;
			args.get(i, tmp);
			startFrame = atoi(tmp.asChar());
			
		}
		else if ( arg == "-end" )
		{
			if (i == args.length()-1)
			{
				arg += ": must specify an end frame";
				displayError(arg);
				return MS::kFailure;
			}
			++i;
			args.get(i, tmp);
			endFrame = atoi(tmp.asChar());
		}
		else
		{
			arg += ": unknown argument";
			displayError(arg);
			return MS::kFailure;
		}
	}

	// Make sure directory name ends with a slash
	if (outDir.substring(outDir.length() - 1, outDir.length() - 1) != "/")
		outDir += "/";

	if (endFrame <= startFrame)
	{
		printf("Start Frame must be before end frame!");
		return MS::kFailure;
	}


	//std::set<float> times = findFrames(startFrame,endFrame);
	skelchar * skel = loadSkeleton(rootJoint, startFrame *(1000.0f/24.0f), endFrame *(1000.0f/24.0f));
	writeSkel((outDir + skelfile).asChar(), skel);
	writeAnim(skel, (outDir + animfile).asChar());
	fflush(stdout);
	//clean up	
	delete skel->joints;
	delete skel->norms;
	delete skel->polys;
	delete skel->verts;
	delete skel->uvs;
	delete skel;

	return MS::kSuccess;
}

